/* ColoredSquare class
 *
 * CMSC498B
 * Project 4
 * April 25, 2002
 *
 * Chad Ellinger
 * Developed with JDK 1.3.1_02 in Windows XP
 *
 * This class is adapted from Sun's Java Tutorials.
 */

import java.awt.*;
import javax.swing.*;

/**
 * A <code>ColoredSquare</code> object is a 3D rectangle icon of a specified color.
 * Big icons are 24x24, small icons are 12x12.
 */
public class ColoredSquare implements Icon{
    Color color;
    boolean big = false;

    /**
     * Constructs a new ColoredSquare with the given color.
     *
     * @param c Color of new ColoredSquare.
     * @param big True if ColoredSquare is big icon, false otherwise.
     */
    public ColoredSquare(Color c, boolean big){
	this.color = c;
	this.big = big;
    }

    /**
     * Constructs a new ColoredSquare with the same color as given ColoredSquare
     *
     * @param cs ColoredSquare to copy.
     * @param big True if ColoredSquare is big icon, false otherwise.
     */
    public ColoredSquare(ColoredSquare cs, boolean big){
	this.color = cs.getColor();
	this.big = big;
	return;
    }

    /**
     * Returns the color of ColoredSquare.
     */
    public Color getColor(){
	return this.color;
    }

    /**
     * Overriden. Draws a 3D Rectangle of current icon width and height with current color.
     */
    public void paintIcon(Component c, Graphics g, int x, int y){
	Color oldColor = g.getColor();
	g.setColor(color);
	g.fill3DRect(x, y, getIconWidth(), getIconHeight(), true);
	g.setColor(oldColor);
    }
    
    /**
     * Returns the width of ColoredSquare.
     */
    public int getIconWidth(){
	if (big){
	    return 24;
	}
	return 12;
    }

    /**
     * Returns the height of ColoredSquare.
     */
    public int getIconHeight(){
	if (big){
	    return 24;
	}
	return 12;
    }
}
