/* Drawsteroids class
 *
 * CMSC498B
 * Project 4
 * April 25, 2002
 *
 * Chad Ellinger
 * Developed with JDK 1.3.1_02 in Windows XP
 */

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.*;

/**
 * The <code>Drawsteroids</code> object creates a new JFrame containing
 * File and Help menus, a toolbar for switching between drawing and game
 * modes, and a drawing palette for creating new objects in drawing mode.
 *
 * @author Chad Ellinger
 */
public class Drawsteroids extends JFrame{
    GameWindow gWindow = new GameWindow();
    JToggleButton gameButton = new JToggleButton(new ImageIcon("running.gif"));
    JToggleButton drawButton = new JToggleButton(new ImageIcon("arrow.gif"));
    JToggleButton arrowButton = new JToggleButton(new ImageIcon("arrow.gif"));
    JToggleButton squareButton = new JToggleButton(new ImageIcon("square.gif"));
    JToggleButton ellipseButton = new JToggleButton(new ImageIcon("ellipse.gif"));
    JToggleButton asteroidButton = new JToggleButton(new ImageIcon("asteroidbutton.gif"));
    JButton colorMenuButton = new JButton();
    boolean drawing = false, gameOver = false;
    Color[] colorList = {Color.magenta, Color.pink, Color.red, Color.orange,
			 Color.yellow, Color.green, Color.blue, Color.white};
    String[] colorNames = {"Magenta", "Pink", "Red", "Orange",
			   "Yellow", "Green", "Blue", "White"};
    ColoredSquare[] squares = new ColoredSquare[colorList.length];
    JMenuItem[] colorMenuItems = new JMenuItem[colorList.length];
    JPopupMenu colorMenu = new JPopupMenu();
    JToolBar drawToolBar = new JToolBar(SwingConstants.VERTICAL);
    JSplitPane splitPane;
    JFrame instructionFrame = new JFrame();

    /**
     * Constructs a new JFrame with a menubar, toolbar and splitpane with drawing toolbar and game window.
     */
    public Drawsteroids(){
	setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	setTitle("Drawsteroids");

	/* Build help frame */
	JEditorPane instructPane = new JEditorPane();
	try{
	    URL instructURL = new URL("file:" + System.getProperty("user.dir") +
			  System.getProperty("file.separator") + "instruct.htm");
	    instructPane.setPage(instructURL);
	}
	catch (MalformedURLException e){
	    System.out.println("Unable to load instructions");
	}
	catch (IOException io){
	    System.out.println("Error creating instruction window");
	}
	instructPane.setPreferredSize(new Dimension(400, 400));
	instructPane.setEditable(false);
	JScrollPane scrollPane = new JScrollPane(instructPane);
	scrollPane.setPreferredSize(new Dimension(400, 400));
	instructionFrame.getContentPane().setLayout(new BorderLayout());
	instructionFrame.getContentPane().add(scrollPane, BorderLayout.CENTER);
	instructionFrame.setTitle("Instructions for Drawsteroids");
	instructionFrame.pack();

	/* Build menu */
	JMenuBar menuBar = new JMenuBar();
	setJMenuBar(menuBar);
	JMenu fileMenu = new JMenu("File");
	fileMenu.setMnemonic(KeyEvent.VK_F);
	menuBar.add(fileMenu);
	JMenu helpMenu = new JMenu("Help");
	helpMenu.setMnemonic(KeyEvent.VK_H);
	menuBar.add(helpMenu);
	JMenuItem newGameItem = new JMenuItem("New Game", KeyEvent.VK_N);
	newGameItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
	newGameItem.addActionListener(new AbstractAction(){
		public void actionPerformed(ActionEvent e){
		    gameOver = false;
		    arrowButton.setSelected(false);
		    squareButton.setSelected(false);
		    ellipseButton.setSelected(false);
		    asteroidButton.setSelected(false);
		    drawButton.setSelected(false);
		    gameButton.setSelected(true);
		    arrowButton.setEnabled(true);
		    squareButton.setEnabled(true);
		    ellipseButton.setEnabled(true);
		    asteroidButton.setEnabled(true);
		    drawButton.setEnabled(true);
		    gameButton.setEnabled(true);
		    gWindow.startNewGame();
		    gWindow.requestFocus();
		}
	    });
	fileMenu.add(newGameItem);
	JMenuItem exitItem = new JMenuItem("Exit", KeyEvent.VK_X);
	exitItem.addActionListener(new AbstractAction(){
		public void actionPerformed(ActionEvent e){
		    System.exit(0);
		}
	    });
	fileMenu.add(exitItem);
	JMenuItem instructItem = new JMenuItem("Instructions", KeyEvent.VK_I);
	instructItem.addActionListener(new AbstractAction(){
		public void actionPerformed(ActionEvent e){
		    instructionFrame.show();
		}
	    });
	helpMenu.add(instructItem);
	JMenuItem aboutItem = new JMenuItem("About", KeyEvent.VK_A);
	aboutItem.addActionListener(new AbstractAction(){
		public void actionPerformed(ActionEvent e){
		    String aboutText = "Drawsteroids\nby Chad Ellinger\n\nCopyright 2002";
		    JOptionPane.showMessageDialog(Drawsteroids.this, aboutText,
						  "About Drawsteroids", JOptionPane.INFORMATION_MESSAGE);
		}
	    });
	helpMenu.add(aboutItem);
	
	getContentPane().setLayout(new BorderLayout());

	/* Build toolbar */
	JToolBar toolBar = new JToolBar();
	toolBar.setFloatable(false);
	gameButton.addActionListener(new AbstractAction(){
		public void actionPerformed(ActionEvent e){
		    toggleDrawMode(false);
		    gWindow.setCursor(Cursor.getDefaultCursor());
		}
	    });
	drawButton.addActionListener(new AbstractAction(){
		public void actionPerformed(ActionEvent e){
		    toggleDrawMode(true);
		    gWindow.setDrawMode(GameWindow.SELECTMODE);
		    gWindow.setCursor(Cursor.getDefaultCursor());
		}
	    });
	gameButton.setSelected(true);
	gameButton.setPreferredSize(new Dimension(31, 31));
	gameButton.setMaximumSize(gameButton.getPreferredSize());
	drawButton.setPreferredSize(new Dimension(31, 31));
	drawButton.setMaximumSize(drawButton.getPreferredSize());
	gameButton.setToolTipText("Game mode");
	drawButton.setToolTipText("Draw mode");
	toolBar.add(gameButton);
	toolBar.add(drawButton);
	getContentPane().add(toolBar, BorderLayout.NORTH);

	/* Create drawing button toolbar */
	drawToolBar.setFloatable(false);
	arrowButton.addActionListener(new AbstractAction(){
		public void actionPerformed(ActionEvent e){
		    Drawsteroids.this.toggleDrawMode(true);
		    gWindow.setDrawMode(GameWindow.SELECTMODE);
		    arrowButton.setSelected(true);
		    squareButton.setSelected(false);
		    ellipseButton.setSelected(false);
		    asteroidButton.setSelected(false);
		    gWindow.setCursor(Cursor.getDefaultCursor());
		}
	    });
	arrowButton.setEnabled(true);
	arrowButton.setToolTipText("Select objects");
	arrowButton.setPreferredSize(new Dimension(31, 31));
	arrowButton.setMinimumSize(arrowButton.getPreferredSize());
	arrowButton.setMaximumSize(arrowButton.getPreferredSize());
	squareButton.addActionListener(new AbstractAction(){
		public void actionPerformed(ActionEvent e){
		    Drawsteroids.this.toggleDrawMode(true);
		    gWindow.setDrawMode(GameWindow.SQUAREMODE);
		    arrowButton.setSelected(false);
		    squareButton.setSelected(true);
		    ellipseButton.setSelected(false);
		    asteroidButton.setSelected(false);
		    gWindow.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
		}
	    });
	squareButton.setEnabled(true);
	squareButton.setToolTipText("Draw rectangle");
	squareButton.setPreferredSize(new Dimension(31, 31));
	squareButton.setMinimumSize(squareButton.getPreferredSize());
	squareButton.setMaximumSize(squareButton.getPreferredSize());
	ellipseButton.addActionListener(new AbstractAction(){
		public void actionPerformed(ActionEvent e){
		    Drawsteroids.this.toggleDrawMode(true);
		    gWindow.setDrawMode(GameWindow.ELLIPSEMODE);
		    arrowButton.setSelected(false);
		    squareButton.setSelected(false);
		    ellipseButton.setSelected(true);
		    asteroidButton.setSelected(false);
		    gWindow.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
		}
	    });
	ellipseButton.setEnabled(true);
	ellipseButton.setToolTipText("Draw ellipse");
	ellipseButton.setPreferredSize(new Dimension(31, 31));
	ellipseButton.setMinimumSize(ellipseButton.getPreferredSize());
	ellipseButton.setMaximumSize(ellipseButton.getPreferredSize());
	asteroidButton.addActionListener(new AbstractAction(){
		public void actionPerformed(ActionEvent e){
		    Drawsteroids.this.toggleDrawMode(true);
		    gWindow.setDrawMode(GameWindow.ASTEROIDMODE);
		    arrowButton.setSelected(false);
		    squareButton.setSelected(false);
		    ellipseButton.setSelected(false);
		    asteroidButton.setSelected(true);
		    gWindow.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
		}
	    });
	asteroidButton.setEnabled(true);
	asteroidButton.setToolTipText("Draw asteroid");
	asteroidButton.setPreferredSize(new Dimension(31, 31));
	asteroidButton.setMinimumSize(asteroidButton.getPreferredSize());
	asteroidButton.setMaximumSize(asteroidButton.getPreferredSize());
	colorMenuButton.setIcon(new ColoredSquare(gWindow.getGlyphColor(), true));
	for (int i = 0; i < colorList.length; i++){
	    squares[i] = new ColoredSquare(colorList[i], false);
	    colorMenuItems[i] = colorMenu.add(new JMenuItem(colorNames[i]));
	    colorMenuItems[i].setIcon(squares[i]);
	    colorMenuItems[i].addActionListener(new AbstractAction(){
		    public void actionPerformed(ActionEvent e){
			ColoredSquare square = (ColoredSquare)((JMenuItem)e.getSource()).getIcon();
			colorMenuButton.setIcon(new ColoredSquare(square, true));
			gWindow.setGlyphColor(square.getColor());
		    }
		});
	}
	colorMenuButton.setEnabled(true);
	colorMenuButton.setToolTipText("Shape color");
	colorMenuButton.setPreferredSize(new Dimension(31, 31));
	colorMenuButton.setMinimumSize(colorMenuButton.getPreferredSize());
	colorMenuButton.setMaximumSize(colorMenuButton.getPreferredSize());
	colorMenuButton.addActionListener(new AbstractAction(){
		public void actionPerformed(ActionEvent e){
		    JButton tempButton = (JButton)e.getSource();
		    colorMenu.show(Drawsteroids.this.drawToolBar, tempButton.getX(),
				   tempButton.getY() + tempButton.getHeight());
		}
	    });
	drawToolBar.add(arrowButton);
	drawToolBar.add(squareButton);
	drawToolBar.add(ellipseButton);
	drawToolBar.add(asteroidButton);
	drawToolBar.addSeparator(new Dimension(0, 10));
	drawToolBar.add(colorMenuButton);

	/* Setup GameWindow */
	gWindow.setPreferredSize(new Dimension(400,400));
	
	/* Set up JSplitPane */
	splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, drawToolBar, gWindow);
	getContentPane().add(splitPane, BorderLayout.CENTER);

	/* Add key listener */
	addKeyListener(new KeyAdapter(){
		public void keyPressed(KeyEvent e){
		    if (e.getKeyCode() == KeyEvent.VK_ENTER){
			toggleDrawMode(!drawing);
			if (drawing){
			    gWindow.setDrawMode(GameWindow.SELECTMODE);
			}
			gWindow.setCursor(Cursor.getDefaultCursor());
		    }
		}
	    });
	pack();
	setVisible(true);
	gWindow.requestFocus();
    }

    /**
     * Disables all of the toolbar buttons. Called when the game is over.
     */
    public void setGameOver(){
	gameOver = true;
	arrowButton.setSelected(false);
	squareButton.setSelected(false);
	ellipseButton.setSelected(false);
	asteroidButton.setSelected(false);
	drawButton.setSelected(false);
	gameButton.setSelected(false);
	arrowButton.setEnabled(false);
	squareButton.setEnabled(false);
	ellipseButton.setEnabled(false);
	asteroidButton.setEnabled(false);
	drawButton.setEnabled(false);
	gameButton.setEnabled(false);
	return;
    }

    /**
     * Switches the game between drawing mode and game mode. If the game switches to game mode,
     * the draw palette buttons are deselected, the game button is selected, and the game is
     * unpaused. If the game switches to draw mode, the selection button and draw buttons are
     * selected and the game is paused.
     *
     * @param draw True if switching to draw mode, false if switching to game mode.
     */
    public void toggleDrawMode(boolean draw){
	if (!gameOver){
	    drawing = draw;
	    if (draw){
		gWindow.pauseGame();
		arrowButton.setSelected(true);
		squareButton.setSelected(false);
		ellipseButton.setSelected(false);
		asteroidButton.setSelected(false);
		drawButton.setSelected(true);
		gameButton.setSelected(false);
	    }
	    else{
		gWindow.requestFocus();
		gWindow.unpauseGame();
		drawButton.setSelected(false);
		gameButton.setSelected(true);
		arrowButton.setSelected(false);
		squareButton.setSelected(false);
		ellipseButton.setSelected(false);
		asteroidButton.setSelected(false);
	    }
	    gWindow.setDrawing(draw);
	}
	return;
    }

    /**
     * Allows Drawsteroids frame to be launched from command line
     */
    public static void main(String[] args){
	new Drawsteroids();
    }
}
