/* GameWindow class
 *
 * CMSC498B
 * Project 4
 * April 25, 2002
 *
 * Chad Ellinger
 * Developed with JDK 1.3.1_02 in Windows XP
 */

import java.applet.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import java.net.*;
import javax.swing.*;

/**
 * A <code>GameWindow</code> object creates a JComponent that keeps track of and
 * draws a ship and multiple glyphs and bullets. The GameWindow creates a Timer that
 * checks for collisions and repaints the screen.
 */
public class GameWindow extends JComponent{
    public final static int SELECTMODE = 0, SQUAREMODE = 1, ELLIPSEMODE = 2, ASTEROIDMODE = 3;
    AudioClip bulletSound, explosionSound, rocketSound;
    ArrayList glyphList = new ArrayList();
    ArrayList bulletList = new ArrayList();
    ArrayList explosionList = new ArrayList();
    Ship ship;
    Image asteroidImage;
    Color[] colorList = {Color.magenta, Color.pink, Color.red, Color.orange,
			 Color.yellow, Color.green, Color.blue, Color.white};
    Color currentGlyphColor = Color.white;
    boolean drawing = false, objectFound = false, gameOver = false;
    int objectSelectedIndex = -1;
    double selectedOffsetX = 0, selectedOffsetY = 0;
    int minWidth = 20, minHeight = 20;
    int drawMode = SELECTMODE;
    int tempX1 = -1, tempX2 = -1, tempY1 = -1, tempY2 = -1;
    int score = 0, level = 0;
    Action updateScreen = new AbstractAction(){
	    public void actionPerformed(ActionEvent e){
		int i;
		if (!drawing){
		    /* Check if any bullets have collided with asteroids */
		    boolean collisionDetected = false;
		    for (i = 0; i < bulletList.size(); i++){
			Bullet bullet = (Bullet)bulletList.get(i);
			int j = 0;
			while ((j < glyphList.size()) && (!collisionDetected)){
			    if (((Glyph)glyphList.get(j)).collision(bullet.getArea())){
				splitGlyph((Glyph)glyphList.get(j), bullet.getDirection());
				glyphList.remove(j);
				collisionDetected = true;
			    }
			    j++;
			}
			if (collisionDetected){
			    bulletList.remove(i);
			    explosionSound.play();
			    score += 100;
			}
		    }
		    /* Check if any asteroids have collided with the ship */
		    for (i = 0; i < glyphList.size(); i++){
			if (((Glyph)glyphList.get(i)).collision(ship.getArea())){
			    pauseGame();
			    gameOver = true;
			    ((Drawsteroids)GameWindow.this.getTopLevelAncestor()).setGameOver();
			    gameTimer.stop();
			}
		    }
		    /* Check if any bullets have traveled max distance */
		    for (i = 0; i < bulletList.size(); i++){
			Bullet bullet = (Bullet)bulletList.get(i);
			if (bullet.getDistanceTraveled() > (int)(0.75*Math.min(getWidth(), getHeight()))){
			    bulletList.remove(i);
			}
		    }
		    /* Check if explosions have expired */
		    for (i = 0; i < explosionList.size(); i++){
			Bullet bullet = (Bullet)explosionList.get(i);
			if (bullet.getDistanceTraveled() > 50){
			    explosionList.remove(i);
			}
		    }
		}
		if (glyphList.isEmpty()){
		    resetGlyphs();
		}
		repaint();
	    }
	};
    public javax.swing.Timer gameTimer = new javax.swing.Timer(50, updateScreen);

    /**
     * Creates a new GameWindow and starts the timer.
     */
    public GameWindow(){
	ship = new Ship();

	/* Load images */
	asteroidImage = Toolkit.getDefaultToolkit().createImage("asteroid.gif");
	MediaTracker tracker = new MediaTracker(this);
	tracker.addImage(asteroidImage, 0);
	try{
	    tracker.waitForID(0);
	}
	catch (InterruptedException e){
	    System.out.println("Couldn't load asteroid.gif");
	    System.exit(0);
	}

	/* Load sounds */
	try{
	    URL bulletSoundURL = new URL("file:" + System.getProperty("user.dir") + System.getProperty("file.separator")
					 + "Blaster01.wav");
	    bulletSound = Applet.newAudioClip(bulletSoundURL);
	}
	catch (MalformedURLException e){
	    System.out.println("Unable to load blaster sound");
	}
	try{
	    URL explosionSoundURL = new URL("file:" + System.getProperty("user.dir")
					    + System.getProperty("file.separator") + "explosion6.wav");
	    explosionSound = Applet.newAudioClip(explosionSoundURL);
	}
	catch (MalformedURLException e){
	    System.out.println("Unable to load explosion sound");
	}
	try{
	    URL rocketSoundURL = new URL("file:" + System.getProperty("user.dir")
					 + System.getProperty("file.separator") + "jetPackLoop.wav");
	    rocketSound = Applet.newAudioClip(rocketSoundURL);
	}
	catch (MalformedURLException e){
	    System.out.println("Unable to load rocket sound");
	}

	addKeyListener(new KeyAdapter(){
		public void keyPressed(KeyEvent e){
		    if (!gameOver && !drawing){
			if (e.getKeyCode() == KeyEvent.VK_RIGHT){
			    ship.setRotateSpeed(Math.PI * 0.67);
			}
			else if (e.getKeyCode() == KeyEvent.VK_LEFT){
			    ship.setRotateSpeed(-1*Math.PI * 0.67);
			}
			else if (e.getKeyCode() == KeyEvent.VK_UP){
			    ship.setAccel(true);
			    rocketSound.loop();
			}
			else if (e.getKeyCode() == KeyEvent.VK_SPACE){
			    Bullet bullet = new Bullet();
			    bullet.setX(ship.getX() + ship.getWidth() / 2);
			    bullet.setY(ship.getY() + ship.getHeight() / 2);
			    bullet.setDirection(ship.getAngle());
			    bulletList.add(bullet);
			    bulletSound.play();
			}
		    }
		}

		public void keyReleased(KeyEvent e){
		    if (e.getKeyCode() == KeyEvent.VK_UP){
			ship.setAccel(false);
			rocketSound.stop();
		    }
		    else if ((e.getKeyCode() == KeyEvent.VK_RIGHT) ||
			     (e.getKeyCode() == KeyEvent.VK_LEFT)){
			ship.setRotateSpeed(0);
		    }			
		}
	    });

	addMouseListener(new MouseAdapter(){
		public void mousePressed(MouseEvent e){
		    if (drawing){
			int i = 0;
			if (drawMode == SELECTMODE){
			    objectFound = false;
			    objectSelectedIndex = -1;
			    ship.setSelected(false);
			    for (i = 0; i < glyphList.size(); i++){
				((Glyph)glyphList.get(i)).setSelected(false);
			    }
			    if (ship.getArea().contains(e.getX(), e.getY())){
				objectFound = true;
				ship.setSelected(true);
				selectedOffsetX = e.getX() - ship.getX();
				selectedOffsetY = e.getY() - ship.getY();
			    }
			    i = glyphList.size() - 1;
			    while ((!objectFound) && (i >= 0)){
				if (((Glyph)glyphList.get(i)).getArea().contains(e.getX(), e.getY())){
				    objectFound = true;
				    objectSelectedIndex = i;
				    selectedOffsetX = e.getX() - ((Glyph)glyphList.get(i)).getX();
				    selectedOffsetY = e.getY() - ((Glyph)glyphList.get(i)).getY();
				    ((Glyph)glyphList.get(i)).setSelected(true);
				}
				i--;
			    }
			}
			else{
			    tempX1 = tempX2 = tempY1 = tempY2 = -1;
			    tempX1 = e.getX();
			    tempY1 = e.getY();
			}
		    }
		}

		public void mouseReleased(MouseEvent e){
		    if (drawing){
			if (drawMode != SELECTMODE){
			    if ((tempX1 != -1) &&
				(tempX2 != -1)){
				int minX = Math.min(tempX1, tempX2);
				int minY = Math.min(tempY1, tempY2);
				int maxX = Math.max(tempX1, tempX2);
				int maxY = Math.max(tempY1, tempY2);
				Glyph newGlyph = new Glyph();
				if (drawMode == SQUAREMODE){
				    newGlyph = new Square(maxX - minX, maxY - minY);
				}
				else if (drawMode == ELLIPSEMODE){
				    newGlyph = new Ellipse(maxX - minX, maxY - minY);
				}
				else if (drawMode == ASTEROIDMODE){
				    int width = Math.min(maxX - minX, maxY - minY);
				    minX = Math.min(tempX1, maxX - width);
				    minY = Math.min(tempY1, maxY - width);
				    newGlyph = new Asteroid(width, width, asteroidImage, GameWindow.this);
				}
				newGlyph.setColor(currentGlyphColor);
				newGlyph.setX(minX);
				newGlyph.setY(minY);
				newGlyph.setDirection(Math.random() * Math.PI * 2);
				newGlyph.setRotateSpeed(Math.random() * (Math.PI / 6));
				newGlyph.setMoving(false);
				glyphList.add(newGlyph);
				tempX1 = tempX2 = tempY1 = tempY2 = -1;
			    }
			}
		    }
		}

		public void mouseEntered(MouseEvent e){
		    if ((drawing) &&
			(drawMode != SELECTMODE)){
			GameWindow.this.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
		    }
		    else{
			GameWindow.this.setCursor(Cursor.getDefaultCursor());
		    }
		}
	    });

	addMouseMotionListener(new MouseMotionAdapter(){
		public void mouseDragged(MouseEvent e){
		    if (drawing){
			if (drawMode == SELECTMODE){
			    if (ship.getSelected()){
				ship.setX(e.getX() - selectedOffsetX);
				ship.setY(e.getY() - selectedOffsetY);
			    }
			    else if (objectSelectedIndex != -1){
				((Glyph)glyphList.get(objectSelectedIndex)).setX(e.getX() - selectedOffsetX);
				((Glyph)glyphList.get(objectSelectedIndex)).setY(e.getY() - selectedOffsetY);
			    }
			}
			else{
			    tempX2 = e.getX();
			    tempY2 = e.getY();
			}
		    }
		}
	    });
	startNewGame();
	gameTimer.start();
    }

    /**
     * Creates a new explosion.
     *
     * @param explosionPoint The point at which to create the explosion.
     */
    public void createExplosion(Point2D.Double explosionPoint){
	double direction = Math.random() * Math.PI / 6;
	for (int i = 1; i < 13; i++){
	    direction += Math.PI / 6;
	    Bullet tempBullet = new Bullet();
	    tempBullet.setLinearSpeed(50);
	    tempBullet.setDirection(direction);
	    tempBullet.setX(explosionPoint.getX());
	    tempBullet.setY(explosionPoint.getY());
	    explosionList.add(tempBullet);
	}
	return;
    }

    /**
     * Returns the current drawing mode.
     */
    public int getDrawMode(){
	return drawMode;
    }

    /**
     * Returns the color that new Glyphs will be drawn with.
     */
    public Color getGlyphColor(){
	return currentGlyphColor;
    }

    /**
     * Overriden. Draws all of the Glyph objects, the ship, and the score and current level.
     */
    public void paintComponent(Graphics g){
	Graphics2D g2 = (Graphics2D)g;
	AffineTransform at = new AffineTransform();
	FontMetrics fontMetrics;
	g2.setColor(Color.black);
	g2.fillRect(0,0,getWidth(),getHeight());
	if (!gameOver){
	    ship.setScreenWidth(getWidth());
	    ship.setScreenHeight(getHeight());
	    at.setToIdentity();
	    g2.setTransform(at);
	    ship.draw(g2);
	    int i;
	    for (i = 0; i < glyphList.size(); i++){
		((Glyph)glyphList.get(i)).setScreenWidth(getWidth());
		((Glyph)glyphList.get(i)).setScreenHeight(getHeight());
		at.setToIdentity();
		g2.setTransform(at);
		((Glyph)glyphList.get(i)).draw(g2);
	    }
	    for (i = 0; i < bulletList.size(); i++){
		((Glyph)bulletList.get(i)).setScreenWidth(getWidth());
		((Glyph)bulletList.get(i)).setScreenHeight(getHeight());
		at.setToIdentity();
		g2.setTransform(at);
		((Glyph)bulletList.get(i)).draw(g2);
	    }
	    for (i = 0; i < explosionList.size(); i++){
		((Glyph)explosionList.get(i)).setScreenWidth(getWidth());
		((Glyph)explosionList.get(i)).setScreenHeight(getHeight());
		at.setToIdentity();
		g2.setTransform(at);
		((Glyph)explosionList.get(i)).draw(g2);
	    }
	}
	if (gameOver){
	    at.setToIdentity();
	    g2.setTransform(at);
	    g2.setColor(Color.red);
	    Font gameOverFont = new Font("Arial", Font.BOLD, 48);
	    g2.setFont(gameOverFont);
	    fontMetrics = g2.getFontMetrics();
	    float stringX = ((float)this.getWidth() / 2) - ((float)fontMetrics.stringWidth("Game Over") / 2);
	    float stringY = ((float)this.getHeight() / 2) + ((float)fontMetrics.getHeight() / 2);
	    g2.drawString("Game Over", stringX, stringY);
	}

	/* Draw score */
	at.setToIdentity();
	g2.setTransform(at);
	g2.setColor(Color.white);
	Font scoreFont = new Font("Arial", Font.BOLD, 18);
	g2.setFont(scoreFont);
	fontMetrics = g2.getFontMetrics();
	String scoreText = "Score: " + score;
	g2.drawString(scoreText, this.getWidth() - fontMetrics.stringWidth(scoreText) - 10, 10 + fontMetrics.getHeight());

	/* Draw level */
	Font levelFont = new Font("Arial", Font.BOLD, 12);
	g2.setFont(levelFont);
	fontMetrics = g2.getFontMetrics();
	String levelText = "Level " + level;
	g2.drawString(levelText, this.getWidth() - fontMetrics.stringWidth(levelText) - 10, this.getHeight() - 10);
	if ((drawing) && (drawMode != 0)){
	    at.setToIdentity();
	    g2.setTransform(at);
	    drawTempGlyph(g2);
	}
	return;
    }

    /**
     * Stops all Glyphs from moving.
     */
    public void pauseGame(){
	int i;
	for (i = 0; i < glyphList.size(); i++){
	    ((Glyph)glyphList.get(i)).setMoving(false);
	}
	for (i = 0; i < bulletList.size(); i++){
	    ((Glyph)bulletList.get(i)).setMoving(false);
	}
	for (i = 0; i < explosionList.size(); i++){
	    ((Glyph)explosionList.get(i)).setMoving(false);
	}
	ship.setMoving(false);
	return;
    }

    /**
     * Clears all glyphs from the screen and creates new Glyphs with random Dimensions and speeds
     * based on current level.
     */
    public void resetGlyphs(){
	level++;
	glyphList.clear();
	bulletList.clear();
	int i;
	int glyphWidth, glyphHeight;
	for (i = 0; i < level; i++){
	    glyphWidth = Math.max(minWidth, (int)(Math.random()*100));
	    glyphHeight = Math.max(minHeight, (int)(Math.random()*100));
	    Asteroid tempAsteroid = new Asteroid(glyphWidth, glyphHeight, asteroidImage, this);
	    initGlyph(tempAsteroid);
	    glyphList.add(tempAsteroid);
	}
	for (i = 0; i < (level - 2); i++){
	    glyphWidth = Math.max(minWidth, (int)(Math.random()*100));
	    glyphHeight = Math.max(minHeight, (int)(Math.random()*100));
	    Ellipse tempEllipse = new Ellipse(glyphWidth, glyphHeight);
	    tempEllipse.setColor(colorList[(int)(Math.random()*colorList.length)]);
	    initGlyph(tempEllipse);
	    glyphList.add(tempEllipse);
	}
	for (i = 0; i < (level - 4); i++){
	    glyphWidth = Math.max(minWidth, (int)(Math.random()*100));
	    glyphHeight = Math.max(minHeight, (int)(Math.random()*100));
	    Square tempSquare = new Square(glyphWidth, glyphHeight);
	    tempSquare.setColor(colorList[(int)(Math.random()*colorList.length)]);
	    initGlyph(tempSquare);
	    glyphList.add(tempSquare);
	}
	return;
    }

    /**
     * Sets the color for new Glyphs to be drawn with to given Color.
     *
     * @param newColor New Color for new Glyphs.
     */
    public void setGlyphColor(Color newColor){
	currentGlyphColor = newColor;
	return;
    }

    /**
     * Sets the GameWindow into drawing mode if <code>drawing=true</code>.
     *
     * @param drawing True to set GameWindow into drawing mode, false otherwise.
     */
    public void setDrawing(boolean drawing){
	this.drawing = drawing;
	return;
    }

    /**
     * Sets the GameWindow's draw mode to one of four predefined modes.
     */
    public void setDrawMode(int drawMode){
	this.drawMode = drawMode;
	int i;
	ship.setSelected(false);
	for (i = 0; i < glyphList.size(); i++){
	    ((Glyph)glyphList.get(i)).setSelected(false);
	}
	return;
    }

    /**
     * Splits the given Glyph into two smaller Glyphs if the Glyph is larger than the
     * minimum width and height.
     *
     * @param glyph The Glyph to split.
     * @param bulletDirection The direction of the bullet that has split the Glyph.
     */
    public void splitGlyph(Glyph glyph, double bulletDirection){
	Glyph newGlyph;
	int newGlyphWidth = glyph.getWidth() / 2;
	int newGlyphHeight = glyph.getHeight() / 2;
	if ((glyph.getWidth() <= minWidth) &&
	    (glyph.getHeight() <= minHeight)){
	    createExplosion(glyph.getCenterPoint());
	}
	else{
	    int numGlyphs = 2;
	    newGlyphWidth = Math.max(minWidth, newGlyphWidth);
	    newGlyphHeight = Math.max(minHeight, newGlyphHeight);
	    double direction = bulletDirection + Math.PI * 0.25;
	    double newX = glyph.getCenterPoint().getX() - glyph.getWidth() * 0.4;
	    double newY = glyph.getCenterPoint().getY() - glyph.getHeight() * 0.4;
	    for (int i = 0; i < numGlyphs; i++){
		if (glyph.getClass().equals(Asteroid.class)){
		    newGlyph = new Asteroid(newGlyphWidth, newGlyphWidth, asteroidImage, this);
		}
		else if (glyph.getClass().equals(Square.class)){
		    newGlyph = new Square(newGlyphWidth, newGlyphHeight);
		}
		else{
		    newGlyph = new Ellipse(newGlyphWidth, newGlyphHeight);
		}
		newGlyph.setX(newX);
		newGlyph.setY(newY);
		newGlyph.setLinearSpeed(glyph.getLinearSpeed());
		newGlyph.setRotateSpeed(Math.random() * Math.PI / 6);
		newGlyph.setColor(glyph.getColor());
		newGlyph.setDirection(direction);
		glyphList.add(newGlyph);
		newX += glyph.getWidth() * 0.2;
		newY += glyph.getHeight() * 0.2;
		direction -= Math.PI * 0.5;
	    }
	}
	return;
    }

    /**
     * Resets the score, level and ship position. Begins a new game and resets game timer.
     */
    public void startNewGame(){
	gameOver = false;
	score = 0;
	level = 0;
	if (this.getWidth() > 0){
	    ship.setX(this.getWidth() / 2 - ship.getWidth() / 2);
	    ship.setY(this.getHeight() / 2 - ship.getHeight() / 2);
	    ship.setScreenWidth(this.getWidth());
	    ship.setScreenHeight(this.getHeight());
	}
	else{
	    ship.setX(200 - ship.getWidth() / 2);
	    ship.setY(200 - ship.getHeight() / 2);
	    ship.setScreenWidth(400);
	    ship.setScreenHeight(400);
	}
	ship.setDirection(Math.PI / -2);
	ship.setMoving(true);
	ship.setLinearSpeed(0);
	resetGlyphs();
	explosionList.clear();
	if (!gameTimer.isRunning()){
	    gameTimer.restart();
	}
	return;
    }

    /**
     * Starts all Glyphs moving.
     */
    public void unpauseGame(){
	int i;
	for (i = 0; i < glyphList.size(); i++){
	    ((Glyph)glyphList.get(i)).setMoving(true);
	}
	for (i = 0; i < bulletList.size(); i++){
	    ((Glyph)bulletList.get(i)).setMoving(true);
	}
	ship.setMoving(true);
	ship.setSelected(false);
	for (i = 0; i < glyphList.size(); i++){
	    ((Glyph)glyphList.get(i)).setSelected(false);
	}
	for (i = 0; i < explosionList.size(); i++){
	    ((Glyph)explosionList.get(i)).setMoving(true);
	}
	return;
    }

    private void drawTempGlyph(Graphics2D g2){
	if ((tempX1 != -1) && (tempX2 != -1)){
	    int minX = Math.min(tempX1, tempX2);
	    int minY = Math.min(tempY1, tempY2);
	    int maxX = Math.max(tempX1, tempX2);
	    int maxY = Math.max(tempY1, tempY2);
	    g2.setColor(Color.white);
	    if (drawMode == SQUAREMODE){
		g2.drawRect(minX, minY, maxX - minX, maxY - minY);
	    }
	    else if (drawMode == ELLIPSEMODE){
		g2.drawOval(minX, minY, maxX - minX, maxY - minY);
	    }
	    else if (drawMode == ASTEROIDMODE){
		int width = Math.min(maxX - minX, maxY - minY);
		minX = Math.min(tempX1, maxX - width);
		minY = Math.min(tempY1, maxY - width);
		g2.drawOval(minX, minY, width, width);
	    }
	}
	return;
    }

    private void initGlyph(Glyph glyph){
	Rectangle2D.Double shipRect = new Rectangle2D.Double(ship.getX() - 50, ship.getY() - 50, 
							     ship.getWidth() + 100, ship.getHeight() + 100);
	double xPoint = Math.random() * getWidth();
	double yPoint = Math.random() * getHeight();
	Area glyphArea = new Area(new Rectangle2D.Double(xPoint, yPoint, glyph.getWidth(), glyph.getHeight()));
	while (glyphArea.intersects(shipRect)){
	    xPoint = Math.random() * getWidth();
	    yPoint = Math.random() * getHeight();
	    glyphArea = new Area(new Rectangle2D.Double(xPoint, yPoint, glyph.getWidth(), glyph.getHeight()));
	}
	glyph.setX(xPoint);
	glyph.setY(yPoint);
	glyph.setDirection(Math.random() * Math.PI * 2);
	glyph.setRotateSpeed(Math.random() * (Math.PI / 6));
	glyph.setLinearSpeed(Math.random() * level * 10);
	return;
    }
}
